unit fROR_XMLEngine;
{
================================================================================
*
*       Package:        ROR - Clinical Case Registries
*       Date Created:   $Revision: 8 $  $Modtime: 4/12/07 3:47p $
*       Site:           Hines OIFO
*       Developers:
*                       Sergey Gavrilov
*
*       Description:    XML engine
*
*       Notes:
*
================================================================================
*       $Archive: /CCR v1.5/Current/fROR_XMLEngine.pas $
*
* $History: fROR_XMLEngine.pas $
 * 
 * *****************  Version 8  *****************
 * User: Zzzzzzgavris Date: 4/12/07    Time: 4:00p
 * Updated in $/CCR v1.5/Current
 * 
 * *****************  Version 7  *****************
 * User: Zzzzzzgavris Date: 3/08/07    Time: 1:47p
 * Updated in $/CCR v1.5/Current
 * 
 * *****************  Version 6  *****************
 * User: Zzzzzzgavris Date: 10/26/05   Time: 11:14a
 * Updated in $/CCR v1.0/Current
 * 
 * *****************  Version 5  *****************
 * User: Zzzzzzgavris Date: 1/10/05    Time: 3:48p
 * Updated in $/CCR v1.0/Current
 * 
 * *****************  Version 4  *****************
 * User: Zzzzzzgavris Date: 10/14/04   Time: 3:50p
 * Updated in $/CCR v1.0/Current
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzgavris Date: 10/21/03   Time: 5:06p
 * Updated in $/ICR v3.0/Current
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzgavris Date: 9/11/03    Time: 10:54p
 * Updated in $/ICR v3.0/Current
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzgavris Date: 9/11/03    Time: 5:03p
 * Created in $/ICR v3.0/Current
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzandria Date: 7/31/03    Time: 12:41p
 * Created in $/ICR v3.0/Current
*
================================================================================
}
interface

uses
  WinApi.Windows, WinApi.Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, SablotEngine, sHandler, TRPCB, uROR_Common, uROR_Broker;

type
  TXMLEngine = class(TForm)
    FSablotEngine: TSablotEngine;
    SchemeProxy: TSchemeProxy;
    procedure FSablotEngineMsgDebug(Sender: TObject;
      const ErrorCode: Integer; var Fields: TSErrorFields);
    procedure FSablotEngineMsgError(Sender: TObject;
      const ErrorCode: Integer; var Fields: TSErrorFields);
    procedure SchemeProxyHasHandle(Sender: TObject; handle: Integer;
      var Result: Boolean);
    procedure SchemeProxyFreeMemory(Sender: TObject; var buffer: PAnsiChar);
    procedure SchemeProxyGet(Sender: TObject; const handle: Integer;
      var buffer: PAnsiChar; var bytecount: Integer);
    procedure SchemeProxyGetAll(Sender: TObject; const scheme,
      rest: PAnsiChar; var buffer: PAnsiChar; var bytecount: Integer);
    procedure SchemeProxyOpen(Sender: TObject; const scheme,
      rest: PAnsiChar; var handle: Integer);
    procedure SchemeProxyPut(Sender: TObject; const handle: Integer;
      const buffer: PAnsiChar; var byteCount: Integer);
    procedure FSablotEngineMsgCritical(Sender: TObject;
      const ErrorCode: Integer; var Fields: TSErrorFields);
    procedure FSablotEngineMsgWarning(Sender: TObject; const ErrorCode: Integer;
      var Fields: TSErrorFields);
  private

    FErrors: TStringList;
    FReportDescr: String;

    FTestXML: integer;
    FTestXSL: integer;

    Broker: TCCRBroker;
    RPCBuf: TStringList;
    RPCTBuf: TStringList;
    Task: String;
    SortMode: TStringList;
    InputBuffer: AnsiString;
    InputEOS: Boolean;
    InputFrom: String;
    OutputStream: TStream;

    tmpbuf: AnsiString;
  public

    property Errors: TStringList read FErrors;
    property ReportDescr: String read FReportDescr;

    constructor Create(aBroker: TCCRBroker); reintroduce;
    destructor  Destroy; override;

    function TransformReport(aTask: String; Output: TStream;
      Template: String = ''; aSortMode: TStringList = nil): Boolean;

  published

    property SablotEngine: TSablotEngine read FSablotEngine;

  end;

implementation
{$R *.DFM}

uses
  uROR_Utilities;

const
  sReportScheme   = 'report';
  sTemplateScheme = 'template';
  sOutputScheme   = 'output';

constructor TXMLEngine.Create(aBroker: TCCRBroker);
begin
  inherited Create(Application);
  Broker := aBroker;
  FReportDescr := 'Report';
  RPCBuf := TStringList.Create;
  RPCTBuf := TStringList.Create;
  OutputStream := nil;
  SortMode := nil;
  FErrors := TStringList.Create;
  FTestXML := 0;
  FTestXSL := 0;
  SchemeProxy.AddScheme(sReportScheme);
  SchemeProxy.AddScheme(sOutputScheme);
  SchemeProxy.AddScheme(sTemplateScheme);
  SchemeProxy.OnFreeMemory := SchemeProxyFreeMemory;
end;

destructor  TXMLEngine.Destroy;
begin
  FErrors.Free;
  RPCBuf.Free;
  RPCTBuf.Free;
  inherited Destroy;
end;

procedure TXMLEngine.FSablotEngineMsgError(Sender: TObject;
  const ErrorCode: Integer; var Fields: TSErrorFields);
begin
  inherited;
  FErrors.Add(DateTimeToStr(now) + ' ' +SablotEngine.LastError);
end;

procedure TXMLEngine.FSablotEngineMsgWarning(Sender: TObject;
  const ErrorCode: Integer; var Fields: TSErrorFields);
begin
  FErrors.Add(DateTimeToStr(now) + ' ' +SablotEngine.LastDebug);
end;

procedure TXMLEngine.FSablotEngineMsgCritical(Sender: TObject;
  const ErrorCode: Integer; var Fields: TSErrorFields);
begin
  FErrors.Add(DateTimeToStr(now) + ' ' +SablotEngine.LastDebug);

end;

procedure TXMLEngine.FSablotEngineMsgDebug(Sender: TObject;
  const ErrorCode: Integer; var Fields: TSErrorFields);
begin
  inherited;
  FErrors.Add(DateTimeToStr(now) + ' ' +SablotEngine.LastDebug);
end;

procedure TXMLEngine.SchemeProxyFreeMemory(Sender: TObject;
 var buffer: PAnsiChar);
begin
  inherited;
  FreeAndNil(buffer);
  Application.ProcessMessages;
end;

procedure TXMLEngine.SchemeProxyHasHandle(Sender: TObject; handle: Integer;
  var Result: Boolean);
begin
  inherited;
  Result := False;
  if (handle = 1) Or (handle = 2) then
    Result := True;
end;

function TXMLEngine.TransformReport(aTask: String; Output: TStream;
  Template: String; aSortMode: TStringList): Boolean;
var
  rptcode: String;
  oldCursor: TCursor;
begin
  Result := False;
  oldCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  try
    FErrors.Clear;
    if Assigned(aSortMode) then
      SortMode := aSortMode;

    if Broker.CallProc(rpcTaskInfo, [aTask]) then
      begin
        rptcode := Piece(Broker.Results[4],'^');
        FReportDescr := Broker.Results[5];
        if FReportDescr = '' then
          FReportDescr := 'Report #' + rptcode;
      end
    else
      rptcode := '';

    if rptcode <> '' then
      begin
        OutputStream := Output;
        OutputStream.Size := 0;
        Task := aTask;
        Result := SablotEngine.RunProcessorGen(
          sTemplateScheme + '://report/' + rptcode + '/' + Template,
          sReportScheme + '://',
          sOutputScheme + '://' );
      end;
  finally
    SortMode := nil;
    Screen.Cursor := oldCursor;
  end;
end;

procedure TXMLEngine.SchemeProxyGet(Sender: TObject; const handle: Integer;
  var buffer: PAnsiChar; var bytecount: Integer);
var
  slen: Integer;
  TestMode: boolean;
  {$IFDEF DBUGETM1}
  DebugETM: TextFile;
  DebugETMisTrue: boolean;
  DebugETMXMLisTrue: boolean;
  {$ENDIF}
begin
  inherited;
  TestMode := (FTestXML > 0);
  {$IFDEF DBUGETM1}
  DebugETMisTrue := FileExists('C:\ETM\DEBUG-ETM-1.txt');
  if DebugETMisTrue then
  begin
    System.AssignFile(DebugETM, 'C:\ETM\DEBUG-ETM-1.txt');
    System.Append(DebugETM);
  end;
  DebugETMXMLisTrue :=  FileExists('C:\ETM\DEBUG-ETM-1.xml');
  TestMode := DebugETMXMLisTrue;
  {$ENDIF}
  if handle = 1 then
    begin
      if Not InputEOS then
        if Broker.CallProc(rpcReportRetrieve,
            [Task,InputFrom,IntToStr(bytecount)+'B'], SortMode, RPCBuf) then
          begin
            InputFrom := RPCBuf.Strings[0];
            if StrToIntDef(Piece(InputFrom,'^'),0) <= 0 then
              InputEOS := True;
            RPCBuf.Delete(0);
            {$IFDEF DBUGETM1}
            if DebugETMXMLisTrue and (FTestXML = 0) then
            begin
              FTestXML := 1;
              RPCBuf.Clear;
              RPCBuf.LoadFromFile('C:\ETM\DEBUG-ETM-1.xml');
             // buffer := RPCBuf.GetText;
             // bytecount := strlen(buffer) + 1;
            end
            else if DebugETMXMLisTrue then
              bytecount := 0;

            if DebugETMisTrue then
            begin
              writeln(DebugETM, '*******In TXMLEngine.SchemeProxyGet******');
              writeln(DebugETM, 'RPC = ' + rpcReportRetrieve);
              writeln(DebugETM, 'RPCBuf=['+RPCBuf.Text+']');
            end;
            {$ENDIF}


            InputBuffer := InputBuffer + RPCBuf.Text;
            RPCBuf.Clear;
          end
        else
          bytecount := 0;

      if bytecount > 0 then
        begin
          slen := Length(InputBuffer);
          if slen > bytecount then
            slen := bytecount
          else
            bytecount := slen;
          Move(InputBuffer[1], buffer, slen);
          Delete(InputBuffer, 1, slen);
        end;
    end;
  {$IFDEF DBUGETM1}
  if DebugETMisTrue then
  begin
    writeln(DebugETM, '*******End of TXMLEngine.SchemeProxyGet******');
    System.CloseFile(DebugETM);
  end;
  {$ENDIF}
end;

procedure TXMLEngine.SchemeProxyGetAll(Sender: TObject; const scheme,
  rest: PAnsiChar; var buffer: PAnsiChar; var bytecount: Integer);
var
  buf, rptcode, template: String;
  TestMode: boolean;
  i, slen: Integer;
 {$IFDEF DBUGETM2}
  DebugETM: TextFile;
  DebugETMisTrue: boolean;
  DebugETMXSLisTrue: boolean;
  {$ENDIF}


begin
  inherited;
  bytecount := 0;
  TestMode  := (FTestXSL > 0);
  tmpbuf    := '';
  {------------------------------------------------
    URL: template://report/5/1
   ------------------------------------------------}
  {$IFDEF DBUGETM2}
  DebugETMisTrue := FileExists('C:\ETM\DEBUG-ETM-2.txt');
  if DebugETMisTrue then
  begin
    System.AssignFile(DebugETM, 'C:\ETM\DEBUG-ETM-2.txt');
    System.Append(DebugETM);
  end;
  {$ENDIF}
  if LowerCase(scheme) = sTemplateScheme then
  begin
    buf := UpperCase(rest);
    if Pos('REPORT/', buf) > 0 then
    begin
      buf := Piece(buf, 'REPORT/', 2);
      rptcode := Piece(buf, '/');
      template := Piece(buf, '/', 2);  //  template
      if Broker.CallProc(rpcReportStylesheet, [rptcode, template], nil, RPCTBuf) then
      begin
        InputFrom := RPCTBuf.Strings[0];
        if StrToIntDef(Piece(InputFrom,'^'),0) <= 0 then
          InputEOS := True;
        //RPCTBuf.Delete(0);

        {$IFDEF DBUGETM2}
        DebugETMXSLisTrue := FileExists('C:\ETM\DEBUG-ETM-2.xsl'+template);
        TestMode := DebugETMXSLisTrue;


        if DebugETMXSLisTrue and (FTestXSL = 0) then
        begin
          FTestXSL := 1;
          RPCTBuf.Clear;
          RPCTBuf.LoadFromFile('C:\ETM\DEBUG-ETM-2.xsl'+template);
          slen := Length(InputBuffer);
          bytecount := slen;
          buffer := AnsiStrAlloc(slen);
          StrPCopy(buffer, Inputbuffer);
          Delete(InputBuffer, 1, slen);
        end
        else if DebugETMXSLisTrue then
          bytecount := 0;

        if DebugETMisTrue then
        begin
          writeln(DebugETM, '*******In TXMLEngine.SchemeProxyGetAll******');
          writeln(DebugETM, 'RPC = ' + rpcReportStylesheet);
          writeln(DebugETM, 'RPCBuf=['+RPCTBuf.Text+']');
        end;
        {$ENDIF}

        for I := 0 to RPCTBuf.count-1 do
        begin
          if Pos('encoding="iso-8859-1"', RPCTBuf.strings[i]) > 0 then
            RPCTBuf.strings[i] := StringReplace(RPCTBuf.strings[i], 'encoding="iso-8859-1"', '',[rfReplaceAll, rfIgnoreCase]);

        end;

        tmpbuf := '';
        InputBuffer := InputBuffer + RPCTBuf.Text;

        if TestMode = false then
        begin
          slen := Length(InputBuffer);
          bytecount := slen;
          buffer := AnsiStrAlloc(slen);
          StrPCopy(buffer, Inputbuffer);
          Delete(InputBuffer, 1, slen);
        end;

      end;
    end;
    RPCTBuf.Clear;
  end;
  {$IFDEF DBUGETM2}
  if DebugETMisTrue then
  begin
    writeln(DebugETM, '*******End of TXMLEngine.SchemeProxyGetAll******');
    System.CloseFile(DebugETM);
  end;
  {$ENDIF}
  Application.ProcessMessages;
end;


procedure TXMLEngine.SchemeProxyOpen(Sender: TObject; const scheme,
  rest: PAnsiChar; var handle: Integer);
begin
  inherited;
  handle := 0;
  if scheme = sReportScheme then
    begin
      InputBuffer := '';
      InputFrom := '';
      InputEOS := False;
      handle := 1;
    end
  else if scheme = sOutputScheme then
    begin
      OutputStream.Size := 0;
      handle := 2;
    end;
end;

procedure TXMLEngine.SchemeProxyPut(Sender: TObject; const handle: Integer;
  const buffer: PAnsiChar; var byteCount: Integer);
begin
  inherited;
  if handle = 2 then
  begin
    if bytecount > 0 then
      OutputStream.WriteBuffer(buffer^, bytecount);
  end;
end;

end.
