unit uROR_MultiStream;

interface

uses
  SysUtils, Classes;

const
  FILESWITCH = #127;  // Character that indicates beginning of a new file

type

  TMultiFileStream = class(TStream)

    fFile: File;
    fFileCount: Integer;
    fFileDir: String;
    fFileExt: String;
    fFileName: String;
    fFileOpen: Boolean;

    function  CloseCurrentFile: Boolean;
    function  GetFileName: String;
    function  NextCount(const Buffer: PAnsiChar; Count: Longint): Longint;
    procedure SwitchFile;

  public

    constructor Create(const aFileName: String);
    destructor Destroy; override;

    function Read(var Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;

  published

    property FileName: String read GetFileName;

  end;

implementation

constructor TMultiFileStream.Create(const aFileName: String);
begin
  fFileDir := ExtractFileDir(aFileName);
  fFileName := ChangeFileExt(ExtractFileName(aFileName), '');
  fFileExt := ExtractFileExt(aFileName);
  fFileCount := 0;
  AssignFile(fFile, FileName);
  Rewrite(fFile, 1);
  fFileOpen := True;
end;

destructor TMultiFileStream.Destroy;
begin
  CloseCurrentFile;
  inherited;
end;

function TMultiFileStream.CloseCurrentFile: Boolean;
var
  fsize: Longint;
begin
  Result := False;
  if fFileOpen then
    begin
      fsize := FileSize(fFile);
      CloseFile(fFile);
      fFileOpen := False;
      if fsize > 0 then
        Result := True
      else
        DeleteFile(FileName);
    end;
end;

function TMultiFileStream.GetFileName: String;
begin
  if fFileCount > 0 then
    Result := Format('%s\%s-%.2d%s',[fFileDir,fFileName,fFileCount,fFileExt])
  else
    Result := Format('%s\%s%s',[fFileDir,fFileName,fFileExt]);
end;

function TMultiFileStream.Read(var Buffer; Count: Longint): Longint;
begin
  Result := 0;
end;

function TMultiFileStream.NextCount(const Buffer: PAnsiChar; Count: Longint): Longint;
var
  bufsize, i: Longint;
begin
  Result := Count;
  bufsize := Count - 1;
  for i:=0 to bufsize do
    if Buffer[i] = FILESWITCH then
      begin
        Result := i;
        Break;
      end;
end;

function TMultiFileStream.Seek(Offset: Longint; Origin: Word): Longint;
var
  pos: Longint;
begin
  pos := FilePos(fFile);
  case Origin of
    soFromBeginning:
      pos := Offset;
    soFromCurrent:
      pos := FilePos(fFile) + Offset;
    soFromEnd:
      pos := FileSize(fFile) + Offset;
  end;
  System.Seek(fFile, pos);
  Result := FilePos(fFile);
end;

procedure TMultiFileStream.SwitchFile;
begin
  if CloseCurrentFile then
    Inc(fFileCount);
  AssignFile(fFile, FileName);
  Rewrite(fFile, 1);
  fFileOpen := True;
end;

function TMultiFileStream.Write(const Buffer; Count: Longint): Longint;
var
  next_count: Longint;
  Base: PAnsiChar;
begin
  Result := 0;
  if Count > 0 then
    begin
      Base := Addr(Buffer);
      next_count := NextCount(Base, Count);
      if next_count > 0 then
        BlockWrite(fFile, Base[0], next_count, Result);
      if next_count < Count then
        begin
          SwitchFile;
          Result := Result + Write(Base[next_count+1], Count-next_count-1) + 1;
        end;
    end;
end;

end.
