unit uROR_User;

interface

uses
  Classes, uROR_Common, uROR_Errors;

type

  TUserInfo = class(TPersistent)
  private
    fIEN:           String;
    fName:          String;
    fNickName:      String;
    fOfficePhone:   String;
    fStationNumber: String;
    fTimeout:       Integer;

  public
    constructor Create;

    function Load(anIEN: String): TReturnCode;

    property IEN:  String               read fIEN;
    property Name: String               read fName;
    property NickName: String           read fNickName;
    property OfficePhone: String        read fOfficePhone;
    property StationNumber: String      read fStationNumber;
    property Timeout: Integer           read fTimeout;

    class function LoadCurrentUserInfo: TReturnCode;

  end;

var
  CCRUser: TUserInfo = nil;

implementation

uses
  Dialogs, SysUtils, uROR_Broker, uROR_Utilities;

constructor TUserInfo.Create;
begin
  inherited;

  fIEN           := '';
  fName          := '';
  fNickName      := '';
  fOfficePhone   := '';
  fStationNumber := '';
  fTimeout       := 600;
end;

function TUserInfo.Load(anIEN: String): TReturnCode;
begin
  try
    if Broker.CallProc(rpcGetUserInfo, [anIEN]) then
      begin
        with Broker do
          begin
            fIEN           := Piece(Results[0], '^', 1);
            fName          := Piece(Results[0], '^', 2);
            fOfficePhone   := Piece(Results[0], '^', 3);
            fNickName      := Piece(Results[0], '^', 4);
            fStationNumber := Piece(Results[0], '^', 10);
            fTimeout       := StrToIntDef(Piece(Results[0], '^', 5), 600);
          end;
        Result := RC_OK;
      end
    else
      Result := RCE_RPC_ERROR;
  except
    on E: Exception do
      begin
        MessageDlg508('', 'Load User Info:'+ #13+#13 + E.Message, mtError, [mbOK], 0);
        Result := RCE_ERROR;
      end;
  end;
end;

class function TUserInfo.LoadCurrentUserInfo: TReturnCode;
begin
  FreeAndNil(CCRUser);
  CCRUser := TUserInfo.Create;
  Result := CCRUser.Load('');
end;

end.
